/***************************************************************************
 *   Copyright (C) 2004 by Stefan Baur                                     *
 *   stefan@c0n.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GLWATER_H
#define GLWATER_H

#define DEFAULT_GL_WATER_REALTIME true
#define DEFAULT_GL_WATER_RES_X 128
#define DEFAULT_GL_WATER_RES_Y 128
#define DEFAULT_GL_WATER_CPS 24
#define DEFAULT_GL_WATER_TIMER_MS 42

#define DEFAULT_GL_WATER_EFFECT_WHIRL true
#define DEFAULT_GL_WATER_EFFECT_BIGBLOB true
#define DEFAULT_GL_WATER_EFFECT_RAIN false




// VBO Extension
//#ifndef GL_ARB_vertex_buffer_object
#ifndef GL_ARRAY_BUFFER_ARB
#define GL_BUFFER_SIZE_ARB                0x8764
#define GL_BUFFER_USAGE_ARB               0x8765
#define GL_ARRAY_BUFFER_ARB               0x8892
#define GL_ELEMENT_ARRAY_BUFFER_ARB       0x8893
#define GL_ARRAY_BUFFER_BINDING_ARB       0x8894
#define GL_ELEMENT_ARRAY_BUFFER_BINDING_ARB 0x8895
#define GL_VERTEX_ARRAY_BUFFER_BINDING_ARB 0x8896
#define GL_NORMAL_ARRAY_BUFFER_BINDING_ARB 0x8897
#define GL_COLOR_ARRAY_BUFFER_BINDING_ARB 0x8898
#define GL_INDEX_ARRAY_BUFFER_BINDING_ARB 0x8899
#define GL_TEXTURE_COORD_ARRAY_BUFFER_BINDING_ARB 0x889A
#define GL_EDGE_FLAG_ARRAY_BUFFER_BINDING_ARB 0x889B
#define GL_SECONDARY_COLOR_ARRAY_BUFFER_BINDING_ARB 0x889C
#define GL_FOG_COORDINATE_ARRAY_BUFFER_BINDING_ARB 0x889D
#define GL_WEIGHT_ARRAY_BUFFER_BINDING_ARB 0x889E
#define GL_VERTEX_ATTRIB_ARRAY_BUFFER_BINDING_ARB 0x889F
#define GL_READ_ONLY_ARB                  0x88B8
#define GL_WRITE_ONLY_ARB                 0x88B9
#define GL_READ_WRITE_ARB                 0x88BA
#define GL_BUFFER_ACCESS_ARB              0x88BB
#define GL_BUFFER_MAPPED_ARB              0x88BC
#define GL_BUFFER_MAP_POINTER_ARB         0x88BD
#define GL_STREAM_DRAW_ARB                0x88E0
#define GL_STREAM_READ_ARB                0x88E1
#define GL_STREAM_COPY_ARB                0x88E2
#define GL_STATIC_DRAW_ARB                0x88E4
#define GL_STATIC_READ_ARB                0x88E5
#define GL_STATIC_COPY_ARB                0x88E6
#define GL_DYNAMIC_DRAW_ARB               0x88E8
#define GL_DYNAMIC_READ_ARB               0x88E9
#define GL_DYNAMIC_COPY_ARB               0x88EA

typedef void (APIENTRY * PFNGLBINDBUFFERARBPROC)    (GLenum target, GLuint buffer);
typedef void (APIENTRY * PFNGLDELETEBUFFERSARBPROC) (GLsizei n, const GLuint *buffers);
typedef void (APIENTRY * PFNGLGENBUFFERSARBPROC)    (GLsizei n, GLuint *buffers);
typedef void (APIENTRY * PFNGLBUFFERDATAARBPROC)    (GLenum target, int size, const GLvoid *data, GLenum usage);

#endif


typedef int WATERTYPE;

typedef struct _VERTEX
	{
	float x,y;
	}VERTEX;

typedef struct _TEXCOORD
	{
	float u,v;
	}TEXCOORD;

typedef unsigned short INDEX;

/**
The platform independent OpenGL water code

@author Stefan Baur
*/
class GLWater
	{
	public:
		GLWater();
		~GLWater();

		void init();
		bool update();
		void render();
		void quit();

		struct
			{
			bool realtime;
			unsigned int bgTexID;
			int water_res_x,water_res_y;
			double water_step_time;
			int timer_ms;

			bool effect_whirl,effect_bigblob,effect_rain;
			}Settings;

		struct
			{
			bool ext_VBO;
			}OpenGLExtensions;

		struct
			{
			PFNGLGENBUFFERSARBPROC glGenBuffersARB;      // VBO Name Generation Procedure
			PFNGLBINDBUFFERARBPROC glBindBufferARB;      // VBO Bind Procedure
			PFNGLBUFFERDATAARBPROC glBufferDataARB;      // VBO Data Loading Procedure
			PFNGLDELETEBUFFERSARBPROC glDeleteBuffersARB;// VBO Deletion Procedure
			}ext;

	private:

		void calcWaterStep();
		bool isExtensionSupported(char *str);
		int waterStep;
		double lastWaterStepTime;

		double lastTick,currentTime,diffTime;
		WATERTYPE *water_data;
		TEXCOORD *water_diff;
		VERTEX *water_vtx;
		TEXCOORD *water_tc;
		INDEX *water_idx;
		unsigned int vbo_vtx,vbo_idx,vbo_tc;
	};

#endif
