/***************************************************************************
 *   Copyright (C) 2004 by Stefan Baur                                     *
 *   stefan@c0n.de                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <stdlib.h>
#include <kapplication.h>
#include <klocale.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qcolor.h>
#include <kurlrequester.h>
#include <kpushbutton.h>
#include <kglobal.h>
#include "kglwatersaver.h"
#include "kglwatersaverui.h"

//! libkscreensaver interface
extern "C"
{
    const char *kss_applicationName = "kglwatersaver.kss";
    const char *kss_description = I18N_NOOP( "KGLWaterSaver" );
    const char *kss_version = "2.2.0";

    KGLWaterSaver *kss_create( WId id )
    {
        KGlobal::locale()->insertCatalogue("kglwatersaver");
        return new KGLWaterSaver( id );
    }

    QDialog *kss_setup()
    {
        KGlobal::locale()->insertCatalogue("kglwatersaver");
        return new KGLWaterSaverSetup();
    }
}

//-----------------------------------------------------------------------------
//! dialog to setup screen saver parameters
KGLWaterSaverSetup::KGLWaterSaverSetup( QWidget *parent, const char *name )
        : KGLWaterSaverUI( parent, name, TRUE )
{
    //Connect your signals and slots here to configure the screen saver.
    connect( OkayPushButton, SIGNAL( released() ),
             SLOT( slotOkPressed() ) );
    connect( CancelPushButton, SIGNAL( released() ),
             SLOT( slotCancelPressed() ) );

	readSettings();
	imageSelected();
}


//! read settings from config file
void KGLWaterSaverSetup::readSettings()
{
	QString img;
    KConfig *config = KGlobal::config();
    config->setGroup( "Settings" );

	cbRealtime->setChecked(config->readBoolEntry( "realtime", DEFAULT_GL_WATER_REALTIME ));
	sbWaterX->setValue(config->readNumEntry( "water_res_x", DEFAULT_GL_WATER_RES_X ));
	sbWaterY->setValue(config->readNumEntry( "water_res_y", DEFAULT_GL_WATER_RES_Y ));
	sbWaterCPS->setValue(config->readNumEntry( "water_cps", DEFAULT_GL_WATER_CPS ));
	sbTimerMS->setValue(config->readNum64Entry( "timer_ms", DEFAULT_GL_WATER_TIMER_MS ));
	img=config->readPathEntry( "background", "desktop" );
	if(img=="desktop")
		{
		rbImageFile->setChecked(false);
		rbImageDesktop->setChecked(true);
		}
	else
		{
		rbImageFile->setChecked(true);
		rbImageDesktop->setChecked(false);
		urlImage->setURL(img);
		}

	cbEffectWhirl->setChecked(config->readBoolEntry( "effect_whirl", DEFAULT_GL_WATER_EFFECT_WHIRL ));
	cbEffectBigBlob->setChecked(config->readBoolEntry( "effect_bigblob", DEFAULT_GL_WATER_EFFECT_BIGBLOB ));
	cbEffectRain->setChecked(config->readBoolEntry( "effect_rain", DEFAULT_GL_WATER_EFFECT_RAIN ));

}


//! Ok pressed - save settings and exit
void KGLWaterSaverSetup::slotOkPressed()
{
	QString img="desktop";

    KConfig *config = KGlobal::config();
    config->setGroup( "Settings" );
	config->writeEntry( "realtime", cbRealtime->isChecked() );
	config->writeEntry( "water_res_x", sbWaterX->value() );
	config->writeEntry( "water_res_y", sbWaterY->value() );
	config->writeEntry( "water_cps", sbWaterCPS->value() );
	config->writeEntry( "timer_ms", sbTimerMS->value() );
	if(rbImageDesktop->isChecked())config->writePathEntry( "background", img);
	else config->writePathEntry( "background", urlImage->url() );

	config->writeEntry( "effect_whirl", cbEffectWhirl->isChecked());
	config->writeEntry( "effect_bigblob", cbEffectBigBlob->isChecked());
	config->writeEntry( "effect_rain", cbEffectRain->isChecked());
    config->sync();

    accept();
}

//! Cancel pressed - exit without saving
void KGLWaterSaverSetup::slotCancelPressed()
{
    reject();
}

//! Image changed - update pixmap and info - FIXME: this is getting called twice a radiobutton is toggled
void KGLWaterSaverSetup::imageSelected()
	{
	QImage img;

	if(rbImageDesktop->isChecked()) //grab desktop
		{
		img=QPixmap::grabWindow(QApplication::desktop()->winId());
		}
	else
		{
		img.load(urlImage->url()); //use file
		}

	if(img.isNull()) //doh! no image found :p TODO: more info
		{
		img.create(8,8,32);
		img.fill(0xFF3311);

		textLabelImageInfo->setText(" - ");
		}
	else
		{
		QString info;
		info.sprintf( "(%ix%i)",img.width(),img.height());
		textLabelImageInfo->setText(info);
		}

	pixImage->setPixmap(img.scale(128,128,QImage::ScaleMin));
	}
//-----------------------------------------------------------------------------



KGLWaterSaver::KGLWaterSaver(WId id):KScreenSaver( id )
	{
	setWFlags(getWFlags()|WNoAutoErase);//FIXME: is this good? We want Qt to do as few updates as possible

	water = new KGLWater; //create "the engine"

	readSettings(); //read in current settings

	embed(water);

	//setBackgroundColor( black ); //FIXME: we don' need this ... do we?
	//erase();

	water->show(); //and get started!
	}

KGLWaterSaver::~KGLWaterSaver()
	{
	delete water; //bye...
	}

//! read configuration settings from config file
void KGLWaterSaver::readSettings()
	{
	KConfig *config = KGlobal::config();
	config->setGroup( "Settings" );
	water->readSettings(config);
	}





